package com.vitorflores.connectfit.Client;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.ScrollView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.ChatsActivity;
import com.vitorflores.connectfit.ContractsActivity;
import com.vitorflores.connectfit.ExercisesActivity;
import com.vitorflores.connectfit.LoginUserActivity;
import com.vitorflores.connectfit.MainActivity;
import com.vitorflores.connectfit.MessageActivity;
import com.vitorflores.connectfit.Model.Contract;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Professional.ClientProfileActivity;
import com.vitorflores.connectfit.R;
import com.vitorflores.connectfit.WorkoutsActivity;

import de.hdodenhof.circleimageview.CircleImageView;

public class ProfessionalProfileActivity extends AppCompatActivity {

    Button mBtnHireProfessional, mBtnMessageProfessional, mBtnWorkouts;
    CircleImageView mImgProfileAvatarView;
    TextView mTextProfileName, mTextProfileLocation, mTextProfileCref, mTextProfileAbout;

    ProgressBar mProgressBar;
    ScrollView mLayoutContainer;

    // Firebase Auth
    FirebaseUser mFirebaseUser;

    private User user;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_professional_profile);

        // Toolbar
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.bar_title_profile);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Don't show the data on the screen until everything is loaded
        mLayoutContainer = findViewById(R.id.layoutContainer);
        mLayoutContainer.setVisibility(View.GONE);

        // Visual elements
        mProgressBar = findViewById(R.id.progressBar);
        mBtnHireProfessional = findViewById(R.id.btnHireProfessional);
        mBtnMessageProfessional = findViewById(R.id.btnMessageProfessional);
        mBtnWorkouts = findViewById(R.id.btnWorkouts);
        mImgProfileAvatarView = findViewById(R.id.imgProfileAvatar);
        mTextProfileName = findViewById(R.id.textProfileName);
        mTextProfileLocation = findViewById(R.id.textProfileLocation);
        mTextProfileCref = findViewById(R.id.textProfileCref);
        mTextProfileAbout = findViewById(R.id.textProfileAbout);

        // Firebase user
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        final Intent intent = getIntent();
        String proId = intent.getStringExtra("id");
        DatabaseReference usersReference = FirebaseDatabase.getInstance().getReference("Users").child(proId);

        // Show the user info on the screen
        usersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                user = dataSnapshot.getValue(User.class);

                // Hide the progress bar and show the profile info to the user
                mProgressBar.setVisibility(View.GONE);
                mLayoutContainer.setVisibility(View.VISIBLE);

                // Show the profile picture of the user
                if (user.getImageURL() != null && user.getImageURL().equals("default")) {
                    mImgProfileAvatarView.setImageResource(R.mipmap.ic_launcher);
                } else {
                    Glide.with(getApplicationContext()).load(user.getImageURL()).into(mImgProfileAvatarView);
                }

                // Show the profile name
                mTextProfileName.setText(user.getUserName());

                // Show the location of the user
                mTextProfileLocation.setVisibility(View.GONE);
                if (user.getCity() != null && user.getState() != null) {
                    if (!user.getCity().equals("") && !user.getState().equals("")) {
                        String location = user.getCity() + "/" + user.getState();
                        mTextProfileLocation.setVisibility(View.VISIBLE);
                        mTextProfileLocation.setText(location);
                    }
                }

                // Show the cref of the user
                mTextProfileCref.setVisibility(View.GONE);
                if (user.getCref() != null) {
                    if (!user.getCref().equals("")) {
                        String cref = "CREF " + user.getCref();
                        mTextProfileCref.setVisibility(View.VISIBLE);
                        mTextProfileCref.setText(cref);
                    }
                }

                // Show the 'about me' section
                mTextProfileAbout.setText(user.getAbout());

                // Show the hire button
                mBtnHireProfessional.setVisibility(View.GONE);
                DatabaseReference contractsReference = FirebaseDatabase.getInstance().getReference("Contracts").child(mFirebaseUser.getUid()).child(proId);
                contractsReference.addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                        if (dataSnapshot.exists()) {
                            Contract contract = dataSnapshot.getValue(Contract.class);

                            assert contract != null;
                            // If the contract is pending, disable the 'hire' button
                            if (contract.getStatus() != null && contract.getStatus().equals("pending")) {
                                mBtnHireProfessional.setText(R.string.action_pending);
                                mBtnHireProfessional.setClickable(false);
                                mBtnHireProfessional.setEnabled(false);
                            } else {
                                // If the contract is active, show the button to end the contract
                                mBtnHireProfessional.setText(R.string.action_dismiss);
                                // Show the 'workouts' button
                                mBtnWorkouts.setVisibility(View.VISIBLE);
                            }
                        }

                        // If there's no contract yet, just shows the default 'hire' button
                        mBtnHireProfessional.setVisibility(View.VISIBLE);
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError databaseError) {}
                });
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });

        mBtnHireProfessional.setOnClickListener(view -> {
            DatabaseReference contractsReference = FirebaseDatabase.getInstance().getReference("Contracts").child(mFirebaseUser.getUid()).child(proId);
            contractsReference.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                    // If there's an active contract and you click to end the contract:
                    if (dataSnapshot.exists()) {
                        contractsReference.removeValue();
                        mBtnHireProfessional.setText(R.string.action_hire);
                    } else {
                        // Send a request to start a contract
                        contractsReference.child("id").setValue(proId);
                        contractsReference.child("status").setValue("pending");
                        mBtnHireProfessional.setText(R.string.action_pending);
                    }

                    contractsReference.removeEventListener(this);
                }

                @Override
                public void onCancelled(@NonNull DatabaseError databaseError) {}
            });

            // The same as above, but on the professional's side
            DatabaseReference contractsReference2 = FirebaseDatabase.getInstance().getReference("Contracts").child(proId).child(mFirebaseUser.getUid());
            contractsReference2.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                    if (dataSnapshot.exists()) {
                        contractsReference2.removeValue();
                    } else {
                        contractsReference2.child("id").setValue(mFirebaseUser.getUid());
                        contractsReference2.child("status").setValue("pending");
                    }

                    contractsReference2.removeEventListener(this);
                }

                @Override
                public void onCancelled(@NonNull DatabaseError databaseError) {}
            });
        });

        mBtnMessageProfessional.setOnClickListener(view -> {
            // Redirects to the chat screen with the user
            Intent intent2 = new Intent(ProfessionalProfileActivity.this, MessageActivity.class);
            intent2.putExtra("id", intent.getStringExtra("id"));
            startActivity(intent2);
        });

        mBtnWorkouts.setOnClickListener(view -> {
            // Redirects to the user workouts
            Intent intent2 = new Intent(ProfessionalProfileActivity.this, WorkoutsActivity.class);
            intent2.putExtra("proId", intent.getStringExtra("id"));
            intent2.putExtra("clientId", mFirebaseUser.getUid());
            startActivity(intent2);
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(ProfessionalProfileActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(ProfessionalProfileActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(ProfessionalProfileActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(ProfessionalProfileActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(ProfessionalProfileActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(ProfessionalProfileActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(ProfessionalProfileActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}