package com.vitorflores.connectfit;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Model.User;

public class LoginUserActivity extends AppCompatActivity {

    private Button mBtnLogin, mBtnRegister;
    private EditText mEmailView, mPasswordView;
    private TextView mForgotPasswordView;

    private FirebaseAuth mAuth;
    private FirebaseUser mFirebaseUser;

    @Override
    protected void onStart() {
        super.onStart();

        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        // Check if logged in
        if (mFirebaseUser != null) {
            Intent intent = new Intent(LoginUserActivity.this, MainActivity.class);
            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(intent);
            finish();
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login_user);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.app_name);

        mBtnLogin = findViewById(R.id.btnLogin);
        mBtnRegister = findViewById(R.id.btnRegister);
        mEmailView = findViewById(R.id.fieldEmail);
        mPasswordView = findViewById(R.id.fieldPassword);
        mForgotPasswordView = findViewById(R.id.btnForgotPassword);

        mAuth = FirebaseAuth.getInstance();

        mBtnLogin.setOnClickListener(v -> {
            String email = mEmailView.getText().toString().toLowerCase();
            String password = mPasswordView.getText().toString();

            if (TextUtils.isEmpty(email) || TextUtils.isEmpty(password)) {
                Toast.makeText(LoginUserActivity.this, R.string.system_message_empty_fields, Toast.LENGTH_SHORT).show();
            } else {
                mAuth.signInWithEmailAndPassword(email, password).addOnCompleteListener(task -> {
                    if (task.isSuccessful()) {
                        // Save the account type on a sharedPreference
                        FirebaseUser firebaseUser = FirebaseAuth.getInstance().getCurrentUser();
                        DatabaseReference usersReference = FirebaseDatabase.getInstance().getReference("Users").child(firebaseUser.getUid());
                        usersReference.addValueEventListener(new ValueEventListener() {
                            @Override
                            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                                User user = dataSnapshot.getValue(User.class);
                                SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
                                SharedPreferences.Editor editor;

                                assert user != null;
                                if (user.getAccountType().equals("pro")) {
                                    editor = accountPrefs.edit();
                                    editor.putString("accountType", "pro");
                                    editor.commit();
                                } else {
                                    editor = accountPrefs.edit();
                                    editor.putString("accountType", "client");
                                    editor.commit();
                                }
                            }

                            @Override
                            public void onCancelled(@NonNull DatabaseError databaseError) {}
                        });

                        // Redirect to the main screen
                        Intent intent = new Intent(LoginUserActivity.this, MainActivity.class);
                        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                        startActivity(intent);
                        finish();
                    } else {
                        Toast.makeText(LoginUserActivity.this, R.string.system_message_login_error, Toast.LENGTH_SHORT).show();
                    }
                });
            }
        });

        mBtnRegister.setOnClickListener(v -> {
            Intent intent = new Intent(LoginUserActivity.this, RegisterUserActivity.class);
            startActivity(intent);
            //finish();
        });

        mForgotPasswordView.setOnClickListener(view -> startActivity(new Intent(LoginUserActivity.this, ResetPasswordActivity.class)));
    }
}