package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Model.Exercise;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Model.Workout;

public class LookUpWorkoutActivity extends AppCompatActivity {

    // Visual Elements
    Button mBtnEditWorkout;
    TextView mTextStatus, mTextDate, mTextWorkout;

    FirebaseUser mFirebaseUser;
    DatabaseReference mReference;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_look_up_workout);
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        Intent intent = getIntent();
        final String name = intent.getStringExtra("name");
        final String proId = intent.getStringExtra("proId");
        final String clientId = intent.getStringExtra("clientId");
        final String workoutId = intent.getStringExtra("id");

        // Toolbar settings
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(name);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        mBtnEditWorkout = findViewById(R.id.btnEditWorkout);
        mTextStatus = findViewById(R.id.textStatus);
        mTextDate = findViewById(R.id.textDate);
        mTextWorkout = findViewById(R.id.textWorkout);

        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        // If you have workouts, add its id to a list
        if (accountType.equals("pro")) {
            mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(mFirebaseUser.getUid()).child(clientId).child(workoutId);
        } else {
            mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(proId).child(mFirebaseUser.getUid()).child(workoutId);
        }
        assert workoutId != null;

        mReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                Workout workout = dataSnapshot.getValue(Workout.class);

                assert workout != null;
                mTextDate.setText(workout.getDate());
                mTextWorkout.setText(workout.getWorkout());

                if (workout.getIsPending()) {
                    mTextStatus.setText("Pendente");
                } else {
                    mTextStatus.setText("Concluído");
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });

        mBtnEditWorkout.setOnClickListener(view -> {
            Intent intent1 = new Intent(LookUpWorkoutActivity.this, EditWorkoutActivity.class);
            intent1.putExtra("proId", proId);
            intent1.putExtra("clientId", clientId);
            intent1.putExtra("workoutId", workoutId);
            startActivity(intent1);
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(LookUpWorkoutActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(LookUpWorkoutActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(LookUpWorkoutActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(LookUpWorkoutActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(LookUpWorkoutActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(LookUpWorkoutActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(LookUpWorkoutActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}