package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Client.AccountClientSettingsActivity;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Professional.AccountProSettingsActivity;

import de.hdodenhof.circleimageview.CircleImageView;

public class MainActivity extends AppCompatActivity {

    Button mBtnProfileSettings, mBtnActiveContracts, mBtnPendingContracts, mBtnProfessionals, mBtnExercises, mBtnChats;
    CircleImageView mImgProfileAvatarView;
    TextView mTextProfileName;
    User user;

    // Firebase Authentication
    private FirebaseAuth mAuth;
    private FirebaseUser mFirebaseUser;
    private DatabaseReference mUsersReference;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.app_name);

        final Boolean isProAccount = false;

        mBtnProfileSettings = findViewById(R.id.btnProfileSettings);
        mBtnActiveContracts = findViewById(R.id.btnActiveContracts);
        mBtnPendingContracts = findViewById(R.id.btnPendingContracts);
        mBtnProfessionals = findViewById(R.id.btnProfessionals);
        mBtnExercises = findViewById(R.id.btnExercises);
        mBtnChats = findViewById(R.id.btnChats);
        mImgProfileAvatarView = findViewById(R.id.imgProfileAvatar);
        mTextProfileName = findViewById(R.id.textProfileName);

        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());

        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                user = dataSnapshot.getValue(User.class);

                assert user != null;
                if (user.getImageURL() != null && user.getImageURL().equals("default")) {
                    mImgProfileAvatarView.setImageResource(R.mipmap.ic_launcher);
                } else {
                    Glide.with(getApplicationContext()).load(user.getImageURL()).into(mImgProfileAvatarView);
                }

                mTextProfileName.setText(user.getUserName());

                // Show the correct buttons based on the type of the account
                mBtnProfileSettings.setVisibility(View.VISIBLE);
                mBtnActiveContracts.setVisibility(View.VISIBLE);
                mBtnPendingContracts.setVisibility(View.VISIBLE);
                mBtnChats.setVisibility(View.VISIBLE);
                mBtnExercises.setVisibility(View.VISIBLE);

                if (!user.getAccountType().equals("pro")) {
                    mBtnProfessionals.setVisibility(View.VISIBLE);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        mBtnProfileSettings.setOnClickListener(view -> {
            Intent intent;

            if (user.getAccountType().equals("pro")) {
                intent = new Intent(MainActivity.this, AccountProSettingsActivity.class);
            } else {
                intent = new Intent(MainActivity.this, AccountClientSettingsActivity.class);
            }

            //intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
            startActivity(intent);
        });

        mBtnActiveContracts.setOnClickListener(view -> {
            Intent intent = new Intent(MainActivity.this, ContractsActivity.class);
            intent.putExtra("typeContracts", "active");
            startActivity(intent);
            //finish();
        });

        mBtnPendingContracts.setOnClickListener(view -> {
            Intent intent = new Intent(MainActivity.this, ContractsActivity.class);
            intent.putExtra("typeContracts", "pending");
            startActivity(intent);
        });

        mBtnProfessionals.setOnClickListener(view -> {
            Intent intent = new Intent(MainActivity.this, AvailableProfessionals.class);
            startActivity(intent);
            //finish();
        });

        mBtnExercises.setOnClickListener(view -> {
            Intent intent = new Intent(MainActivity.this, ExercisesActivity.class);
            startActivity(intent);
        });

        mBtnChats.setOnClickListener(view -> {
            Intent intent = new Intent(MainActivity.this, ChatsActivity.class);
            startActivity(intent);
            //finish();
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu, menu);
        return true;
    }

    // If the logout button is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(MainActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}