package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Adapter.MessageAdapter;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Fragments.APIService;
import com.vitorflores.connectfit.Model.Chat;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Notifications.Client;
import com.vitorflores.connectfit.Notifications.Data;
import com.vitorflores.connectfit.Notifications.Response;
import com.vitorflores.connectfit.Notifications.Sender;
import com.vitorflores.connectfit.Notifications.Token;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import de.hdodenhof.circleimageview.CircleImageView;
import retrofit2.Call;
import retrofit2.Callback;

public class MessageActivity extends AppCompatActivity {

    // Visual elements
    CircleImageView mImgProfileAvatarView;
    EditText mFieldSend;
    ImageButton mBtnSend;
    TextView mTextProfileName;
    User user;

    MessageAdapter messageAdapter;
    List<Chat> mChat;
    RecyclerView messagesLayout;

    // Firebase Authentication
    private FirebaseUser mFirebaseUser;
    private DatabaseReference mUsersReference;

    Intent intent;
    String receiverId;

    // Notifications
    APIService apiService;
    boolean notify = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_message);

        // Toolbar
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle("");
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Establish a communication with the fcm server
        apiService = Client.getClient("https://fcm.googleapis.com/").create(APIService.class);

        // Visual Elements
        mImgProfileAvatarView = findViewById(R.id.imgProfileAvatar);
        mTextProfileName = findViewById(R.id.textProfileName);
        mFieldSend = findViewById(R.id.fieldSend);
        mBtnSend = findViewById(R.id.btnSend);

        messagesLayout = findViewById(R.id.messagesLayout);
        messagesLayout.setHasFixedSize(true);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(getApplicationContext());
        linearLayoutManager.setStackFromEnd(true);
        messagesLayout.setLayoutManager(linearLayoutManager);

        intent = getIntent();
        receiverId = intent.getStringExtra("id");
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        // Send a message
        mBtnSend.setOnClickListener(view -> {
            notify = true;
            String message = mFieldSend.getText().toString();

            if (!message.equals("")) {
                sendMessage(mFirebaseUser.getUid(), receiverId, message);
            } else {
                Toast.makeText(MessageActivity.this, "Error", Toast.LENGTH_SHORT).show();
            }

            mFieldSend.setText("");
        });

        assert receiverId != null;
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(receiverId);

        // Puts the profile picture and name of the receiver in the toolbar
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                user = dataSnapshot.getValue(User.class);

                assert user != null;
                if (user.getImageURL().equals("default")) {
                    mImgProfileAvatarView.setImageResource(R.mipmap.ic_launcher);
                } else {
                    Glide.with(getApplicationContext()).load(user.getImageURL()).into(mImgProfileAvatarView);
                }

                mTextProfileName.setText(user.getUserName());

                // Show the messages with the receiver on the screen
                readMessages(mFirebaseUser.getUid(), receiverId);
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });
    }

    // Send a message
    private void sendMessage(String sender, String receiver, String message) {
        DatabaseReference reference = FirebaseDatabase.getInstance().getReference();

        HashMap<String, java.lang.Object> hashMap = new HashMap<>();
        hashMap.put("sender", sender);
        hashMap.put("receiver", receiver);
        hashMap.put("message", message);

        reference.child("Chats").push().setValue(hashMap);

        // If you send a message to a user, keep their id saved on your chatlist so you can know who you had a chat with
        final DatabaseReference chatSenderReference = FirebaseDatabase.getInstance().getReference("Chatlist")
                .child(mFirebaseUser.getUid())
                .child(receiverId);

        chatSenderReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                if (!dataSnapshot.exists()) {
                    chatSenderReference.child("id").setValue(receiverId);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        // Do the same thing as above but with your id under the receiver's chatlist
        final DatabaseReference chatReceiverReference = FirebaseDatabase.getInstance().getReference("Chatlist")
                .child(receiverId)
                .child(mFirebaseUser.getUid());

        chatReceiverReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                if (!dataSnapshot.exists()) {
                    chatReceiverReference.child("id").setValue(mFirebaseUser.getUid());
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        // After you sent a message, send a notification to the receiver
        final String msg = message;
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                User user = dataSnapshot.getValue(User.class);

                assert user != null;
                if (notify) {
                    sendNotification(receiver, user.getUserName(), msg);
                }

                notify = false;
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });
    }

    // Send a notification to the receiver, after you sent him a message
    private void sendNotification(String receiver, final String username, String message) {
        DatabaseReference tokens = FirebaseDatabase.getInstance().getReference("Tokens");
        Query query = tokens.orderByKey().equalTo(receiver);
        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    Token token = snapshot.getValue(Token.class);
                    Data data = new Data(mFirebaseUser.getUid(), R.mipmap.ic_launcher, username + ": " + message, "Nova Mensagem", receiverId);

                    assert token != null;
                    Sender sender = new Sender(data, token.getToken());

                    apiService.sendNotification(sender)
                            .enqueue(new Callback<Response>() {
                                @Override
                                public void onResponse(Call<Response> call, retrofit2.Response<Response> response) {
                                    if (response.code() == 200) {
                                        assert response.body() != null;
                                        if (response.body().success != 1) {
                                            Toast.makeText(MessageActivity.this, "Error", Toast.LENGTH_SHORT).show();
                                        }
                                    }
                                }

                                @Override
                                public void onFailure(Call<Response> call, Throwable t) {

                                }
                            });
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });
    }

    // Show the messages on the screen
    private void readMessages(final String myId, final String ReceiverId) {
        mChat = new ArrayList<>();

        mUsersReference = FirebaseDatabase.getInstance().getReference("Chats");
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                mChat.clear();

                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    Chat chat = snapshot.getValue(Chat.class);

                    assert chat != null;
                    if ((chat.getReceiver().equals(myId) && chat.getSender().equals(ReceiverId)) ||
                        (chat.getReceiver().equals(ReceiverId) && chat.getSender().equals(myId))) {
                        mChat.add(chat);
                    }

                    // Take the messages list and show all the messages with the other person on the screen
                    messageAdapter = new MessageAdapter(MessageActivity.this, mChat);
                    messagesLayout.setAdapter(messageAdapter);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(MessageActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(MessageActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(MessageActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(MessageActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(MessageActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(MessageActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(MessageActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}