package com.vitorflores.connectfit.Professional;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;

import android.Manifest;
import android.accounts.Account;
import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Looper;
import android.provider.Settings;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.webkit.MimeTypeMap;
import android.widget.Button;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationCallback;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationResult;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.tasks.Continuation;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.StorageTask;
import com.google.firebase.storage.UploadTask;
import com.vitorflores.connectfit.ChatsActivity;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.ContractsActivity;
import com.vitorflores.connectfit.ExercisesActivity;
import com.vitorflores.connectfit.LoginUserActivity;
import com.vitorflores.connectfit.MainActivity;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.R;
import com.vitorflores.connectfit.WorkoutsActivity;

import java.util.HashMap;

import de.hdodenhof.circleimageview.CircleImageView;

public class AccountProSettingsActivity extends AppCompatActivity {

    // Visual elements
    private CircleImageView mImgProfileAvatarView;
    private TextView mTextCref, mTextAbout, mTextCity, mTextState;
    private Button mBtnSave;
    private Switch mSwitchUpdateLocation;

    // Firebase Authentication
    private FirebaseUser mFirebaseUser;
    private DatabaseReference mUsersReference;

    // Image upload
    StorageReference storageReference;
    private static final int IMAGE_REQUEST = 1;
    private Uri mImageUri;
    private StorageTask mUploadTask;

    // Location
    FusedLocationProviderClient fusedLocationProviderClient;

    User user;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_account_pro_settings);

        // Toolbar
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.bar_title_profile_settings);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mImgProfileAvatarView = findViewById(R.id.imgProfileAvatar);
        mTextAbout = findViewById(R.id.fieldAbout);
        mTextCref = findViewById(R.id.fieldCref);
        mTextCity = findViewById(R.id.fieldCity);
        mTextState = findViewById(R.id.fieldState);
        mSwitchUpdateLocation = findViewById(R.id.switchUpdateLocation);
        mBtnSave = findViewById(R.id.btnSave);

        // Firebase Auth
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());

        // Image upload
        storageReference = FirebaseStorage.getInstance().getReference("uploads");

        // Location
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(AccountProSettingsActivity.this);

        // Fill the fields with the current values
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                user = dataSnapshot.getValue(User.class);

                assert user != null;
                if (user.getImageURL() != null && user.getImageURL().equals("default")) {
                    mImgProfileAvatarView.setImageResource(R.mipmap.ic_launcher);
                } else {
                    Glide.with(getApplicationContext()).load(user.getImageURL()).into(mImgProfileAvatarView);
                }

                mTextAbout.setText(user.getAbout());
                mTextCref.setText(user.getCref());
                mTextCity.setText(user.getCity());
                mTextState.setText(user.getState());
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        // Change the profile image
        mImgProfileAvatarView.setOnClickListener(view -> openImage());

        // Update the account details
        mBtnSave.setOnClickListener(v -> {
            final String about = mTextAbout.getText().toString();
            final String cref = mTextCref.getText().toString();
            final String city = mTextCity.getText().toString();
            final String state = mTextState.getText().toString();
            final boolean updateLocation = mSwitchUpdateLocation.isChecked();

            mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());

            // Combine the field values into one hashmap
            HashMap<String, java.lang.Object> hashMap = new HashMap<>();

            // Update location
            if (updateLocation) {
                // Check condition
                if (ActivityCompat.checkSelfPermission(AccountProSettingsActivity.this,
                    Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
                    && ActivityCompat.checkSelfPermission(AccountProSettingsActivity.this,
                    Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {

                    getCurrentLocation();

                } else {
                    // If permission denied, request permission
                    //ActivityCompat.requestPermissions(AccountProSettingsActivity.this,
                    //        new String[]{Manifest.permission.ACCESS_FINE_LOCATION,Manifest.permission.ACCESS_COARSE_LOCATION}, 100);
                    Toast.makeText(AccountProSettingsActivity.this, "Ative a localização nas permissões do aplicativo", Toast.LENGTH_SHORT).show();
                }
            }

            hashMap.put("about", about);
            hashMap.put("cref", cref);
            hashMap.put("city", city);
            hashMap.put("state", state);


            // Update the account details and redirects to the main screen
            updateUserInfo(hashMap);
        });
    }

    /*@Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        // check condition
        if (requestCode == 100 && grantResults.length > 0 && (grantResults[0] + grantResults[1] == PackageManager.PERMISSION_GRANTED)) {
            getCurrentLocation();
        } else {
            // Permission denied
            Toast.makeText(AccountProSettingsActivity.this, "Permissão negada para acessar localização", Toast.LENGTH_SHORT).show();
        }
    }*/

    @SuppressLint("MissingPermission")
    private void getCurrentLocation() {
        LocationManager locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);

        if (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
            || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {

            fusedLocationProviderClient.getLastLocation().addOnCompleteListener(new OnCompleteListener<Location>() {
                @Override
                public void onComplete(@NonNull Task<Location> task) {
                    Location location = task.getResult();

                    if (location != null) {
                        // Set latitude and longitude
                        HashMap<String, java.lang.Object> hashMap = new HashMap<>();
                        hashMap.put("latitude", location.getLatitude());
                        hashMap.put("longitude", location.getLongitude());

                        updateUserLocation(hashMap);
                    } else {
                        LocationRequest locationRequest = new LocationRequest()
                                .setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY)
                                .setInterval(10000)
                                .setFastestInterval(1000)
                                .setNumUpdates(1);

                        LocationCallback locationCallback = new LocationCallback() {
                            @Override
                            public void onLocationResult(LocationResult locationResult) {
                                Location location1 = locationResult.getLastLocation();
                                // Set lat and long
                                HashMap<String, java.lang.Object> hashMap = new HashMap<>();
                                hashMap.put("latitude", location1.getLatitude());
                                hashMap.put("longitude", location1.getLongitude());

                                updateUserLocation(hashMap);
                            }
                        };

                        fusedLocationProviderClient.requestLocationUpdates(locationRequest, locationCallback, Looper.myLooper());
                    }
                }
            });
        } else {
            // When location service is disabled
            //startActivity(new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
            //    .setFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
            Toast.makeText(AccountProSettingsActivity.this, "Ative a localização nas configurações do Android", Toast.LENGTH_SHORT).show();
        }
    }

    private void updateUserLocation(HashMap<String, java.lang.Object> hashMap) {
        mUsersReference.updateChildren(hashMap).addOnCompleteListener(task1 -> {
            if (!task1.isSuccessful()) {
                Toast.makeText(AccountProSettingsActivity.this, "Erro ao atualizar localização", Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void updateUserInfo(HashMap<String, java.lang.Object> hashMap) {
        mUsersReference.updateChildren(hashMap).addOnCompleteListener(task -> {
            if (task.isSuccessful()) {
                Intent intent = new Intent(AccountProSettingsActivity.this, MainActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intent);
                finish();
            } else {
                Toast.makeText(AccountProSettingsActivity.this, "Error", Toast.LENGTH_SHORT).show();
            }
        });
    }

    // Open the screen to choose a new image from the gallery
    private void openImage() {
        Intent intent = new Intent();
        intent.setType("image/*");
        intent.setAction(Intent.ACTION_GET_CONTENT);
        startActivityForResult(intent, IMAGE_REQUEST);
    }

    // Get the extension of a file
    private String getFileExtension(Uri uri) {
        ContentResolver contentResolver = getApplicationContext().getContentResolver();
        MimeTypeMap mimeTypeMap = MimeTypeMap.getSingleton();

        return mimeTypeMap.getExtensionFromMimeType(contentResolver.getType(uri));
    }

    // Upload the new image to the firebase server
    private void uploadImage() {
        // Progress dialog while the image is uploading
        final ProgressDialog pd = new ProgressDialog(AccountProSettingsActivity.this);
        pd.setMessage("Uploading");
        pd.show();

        // Upload the image
        if (mImageUri != null) {
            final StorageReference fileReference = storageReference.child(System.currentTimeMillis()
                    + "." + getFileExtension(mImageUri));

            mUploadTask = fileReference.putFile(mImageUri);
            // If the app failed to upload, show the error message
            // If the image was successful uploaded, update the profile picture with the new image
            mUploadTask.continueWithTask(new Continuation<UploadTask.TaskSnapshot, Task<Uri>>() {
                @Override
                public Task<Uri> then(@NonNull Task<UploadTask.TaskSnapshot> task) throws Exception {
                    if (!task.isSuccessful()) {
                        throw task.getException();
                    }

                    return fileReference.getDownloadUrl();
                }
            }).addOnCompleteListener(task -> {
                // If the image was successful uploaded, update the profile picture with the new image
                if (task.isSuccessful()) {
                    String uri = task.getResult().toString();

                    mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());
                    HashMap<String, java.lang.Object> map = new HashMap<>();
                    map.put("imageURL", uri);
                    mUsersReference.updateChildren(map);
                } else {
                    Toast.makeText(AccountProSettingsActivity.this, "Error", Toast.LENGTH_SHORT).show();
                }

                pd.dismiss();
            }).addOnFailureListener(e -> {
                // If the app failed to upload, show the error message
                Toast.makeText(AccountProSettingsActivity.this, e.getMessage(), Toast.LENGTH_SHORT).show();
                pd.dismiss();
            });
        } else {
            Toast.makeText(AccountProSettingsActivity.this, "No image selected", Toast.LENGTH_SHORT).show();
        }
    }

    // Wait for the user to choose the new image, after that, call the method to upload it to the server
    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == IMAGE_REQUEST && resultCode == RESULT_OK
                && data != null && data.getData() != null) {
            mImageUri = data.getData();

            if (mUploadTask != null && mUploadTask.isInProgress()) {
                Toast.makeText(AccountProSettingsActivity.this, "Upload in progress", Toast.LENGTH_SHORT).show();
            } else {
                uploadImage();
            }
        }
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(AccountProSettingsActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(AccountProSettingsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(AccountProSettingsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(AccountProSettingsActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(AccountProSettingsActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(AccountProSettingsActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(AccountProSettingsActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}