package com.vitorflores.connectfit;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.vitorflores.connectfit.Client.AccountClientSettingsActivity;
import com.vitorflores.connectfit.Professional.AccountProSettingsActivity;

import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;

public class RegisterUserActivity extends AppCompatActivity {

    // Visual Elements
    private EditText mUserNameView, mEmailView, mPasswordView, mConfirmPasswordView;
    private RadioButton mAccountTypeClient, mAccountTypePro;
    private Button mBtnRegister;

    // Firebase Authentication
    private FirebaseAuth mAuth;

    private DatabaseReference mReference;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register_user);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.action_register_account);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        mUserNameView = findViewById(R.id.fieldName);
        mEmailView = findViewById(R.id.fieldEmail);
        mPasswordView = findViewById(R.id.fieldPassword);
        mConfirmPasswordView = findViewById(R.id.fieldConfirmPassword);
        mAccountTypeClient = findViewById(R.id.rbAccountTypeClient);
        mAccountTypePro = findViewById(R.id.rbAccountTypePro);
        mBtnRegister = findViewById(R.id.btnRegister);
        mAuth = FirebaseAuth.getInstance();

        mBtnRegister.setOnClickListener(v -> {
            final String name = mUserNameView.getText().toString();
            final String email = mEmailView.getText().toString().toLowerCase();
            final String password = mPasswordView.getText().toString();
            final String confirmPassword = mConfirmPasswordView.getText().toString();

            if (!name.equals("") && !email.equals("") && !password.equals("") && !confirmPassword.equals("") && password.equals(confirmPassword)) {
                createFirebaseUser();
            } else {
                Toast.makeText(RegisterUserActivity.this, R.string.system_message_register_error, Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void createFirebaseUser() {
        final String name = mUserNameView.getText().toString();
        final String email = mEmailView.getText().toString().toLowerCase();
        final boolean isPro;
        final String password = mPasswordView.getText().toString();

        if (mAccountTypePro.isChecked()) {
            isPro = true;
        } else {
            isPro = false;
        }

        // Register an user
        mAuth.createUserWithEmailAndPassword(email, password).addOnCompleteListener(task -> {
            if (task.isSuccessful()) {
                FirebaseUser firebaseUser = mAuth.getCurrentUser();
                assert firebaseUser != null;
                String userid = firebaseUser.getUid();

                mReference = FirebaseDatabase.getInstance().getReference("Users").child(userid);

                HashMap<String, String> hashMap = new HashMap<>();
                hashMap.put("id", userid);
                hashMap.put("userName", name);
                String searchTerm = StringUtils.stripAccents(name.toLowerCase());
                hashMap.put("searchableName", searchTerm);
                hashMap.put("imageURL", "default");
                hashMap.put("height", "");
                hashMap.put("weight", "");
                hashMap.put("about", "");
                hashMap.put("cref", "");
                hashMap.put("city", "");
                hashMap.put("state", "");

                if (isPro) {
                    hashMap.put("accountType", "pro");
                } else {
                    hashMap.put("accountType", "client");
                }


                // Redirects to the 'edit user' screen
                mReference.setValue(hashMap).addOnCompleteListener(task1 -> {
                    if (task1.isSuccessful()) {
                        mAuth.signInWithEmailAndPassword(email, password).addOnCompleteListener(task11 -> {
                            if (task11.isSuccessful()) {
                                Intent intent;
                                SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
                                SharedPreferences.Editor editor;

                                if (isPro) {
                                    intent = new Intent(RegisterUserActivity.this, AccountProSettingsActivity.class);
                                    // Save the account type on a sharedPref
                                    editor = accountPrefs.edit();
                                    editor.putString("accountType", "pro");
                                    editor.commit();
                                } else {
                                    intent = new Intent(RegisterUserActivity.this, AccountClientSettingsActivity.class);
                                    // Save the account type on a sharedPref
                                    editor = accountPrefs.edit();
                                    editor.putString("accountType", "client");
                                    editor.commit();
                                }

                                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                                startActivity(intent);
                                finish();
                            } else {
                                Toast.makeText(RegisterUserActivity.this, R.string.system_message_login_error, Toast.LENGTH_SHORT).show();
                            }
                        });
                    }
                });
            } else {
                Toast.makeText(RegisterUserActivity.this, R.string.system_message_register_error, Toast.LENGTH_SHORT).show();
            }
        });
    }
}